/**
  ******************************************************************************
  * @file    Simple Servo Control Demo, designed to work with STM32F4 Discovery and Futaba S3003
  * @author  Lee Dyche - Tecsploit.com
  * @version V1.0.0
  * @date    12/04/2015
  * @brief   Uses PWM to control the servo, the servo cycles between 3 positions, 0 degrees, 90 degrees and 180 degrees
  * The servos control wire (white) should be connected to PB6
  ******************************************************************************
  * @attention
  * This is program is provided as is with no warranty!!
  ******************************************************************************
  */

#include "stm32f4_discovery.h"

//constants may change from servo to servo, for my Futaba S3003 I found that these values worked well for the min and max
//rotation.
__IO uint16_t ServoMinAngle = 20;		// 0
__IO uint16_t ServoMaxAngle = 2500;		// 180

void Delay(__IO uint32_t nCount);
void SetupPWM(uint32_t period, uint16_t PrescalerValue);
void SetupPins();
void SetServoPosition(uint32_t angle);
void PositionServoPulse(uint32_t pulse);

int main(void)
{
	  uint32_t prescalar = (uint16_t) ( ( (SystemCoreClock / 2) / 1000000) - 1);

	  SetupPWM(20000, prescalar);


	  while(1){

		  SetServoPosition(0);
		  Delay(50000000);

		  SetServoPosition(90);
		  Delay(50000000);

		  SetServoPosition(180);
		  Delay(50000000);
	  }
}

/*
 * This method positions the servo given an angle in degrees between 0 and 180, it scales the angle between
 * ServoMinAngle and ServoMaxAngle for example a value of 90 will be half way between these values. 0 is set to ServoMinAngle
 * and anything above 180 is set to ServoMaxAngle
 */
void SetServoPosition(uint32_t angle){
	uint32_t range = ServoMaxAngle - ServoMinAngle;
	uint32_t unit = range / 180;

	uint32_t step = unit * angle;

	if(step == 0)
		step = ServoMinAngle;

	if(step > ServoMaxAngle)
		step = ServoMaxAngle;

	PositionServoPulse(step);
}

/*
 * Positions the servo by re-configuring PWM and changing the pulse value to be the value passed in
 * for the Futaba S3003 im using this should be between 20 and 2500
 */
void PositionServoPulse(uint32_t pulse){
	  TIM_OCInitTypeDef  TIM_OCInitStructure;
	/* Configure timer for PWM - channel 1*/
	  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
	  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	  TIM_OCInitStructure.TIM_Pulse = pulse;
	  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

	  //notice the number 1 in TIM_OC1Init
	  TIM_OC1Init(TIM4, &TIM_OCInitStructure);
	  TIM_OC1PreloadConfig(TIM4, TIM_OCPreload_Enable);
}

/**
 * Sets up PWM and the PB6 pin ready for use with an initial value of ServoMinAngle
 */
void SetupPWM(uint32_t period, uint16_t PrescalerValue){
  SetupPins();

	/*Structures used in the configuration*/
  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
  TIM_OCInitTypeDef  TIM_OCInitStructure;

  /* Setup timer defaults */
  TIM_TimeBaseStructure.TIM_Period = period;
  TIM_TimeBaseStructure.TIM_Prescaler = PrescalerValue;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;

  TIM_TimeBaseInit(TIM4, &TIM_TimeBaseStructure);

  /* Configure timer for PWM - channel 1*/
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 0;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

  //notice the number 1 in TIM_OC1Init
  TIM_OC1Init(TIM4, &TIM_OCInitStructure);
  TIM_OC1PreloadConfig(TIM4, TIM_OCPreload_Enable);


  TIM_ARRPreloadConfig(TIM4, ENABLE);

  /* Start timer*/
  TIM_Cmd(TIM4, ENABLE);

}


/**
 * Sets up the PB6 pin for PWM
 */
void SetupPins(){
	/* Enable TIM4 Clock */
	  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);


	  //Enabled GPIOB we are going to use PB6 which is linked to TIM4_CH1 according to the
	  //documentation
	  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);

	  GPIO_InitTypeDef GPIO_InitStructure;
	  //initalise pin 6 B - relating to timer 4 channel 1
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 ;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
	  GPIO_Init(GPIOB, &GPIO_InitStructure);

	  //link the pins up to the timer - AF stands for alternate function
	  GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_TIM4);
}

/**
  * @brief  Delay Function.
  * @param  nCount:specifies the Delay time length.
  * @retval None
  */
void Delay(__IO uint32_t nCount)
{
  while(nCount--)
  {
  }
}


